# -*- mode: perl; -*-
# $Id: xmms_info.pl 52 2005-12-11 22:45:54Z dmw $

use strict;
use vars qw($VERSION %IRSSI);

use Irssi;

$VERSION = '1.1.$Revision: 52 $';
%IRSSI = (
    authors	=> "sickd",
    contact	=> 'dmw@unete.cl',
    name	=> "XMMS/Infopipe Plugin",
    description	=> "Announces XMMS Playing Song using xmms-infopipe",
    license	=> "Public Domain",
    url		=> "http://www.linux-chile.org/docs.php?op=verVersion&doc=17&id=1",
    changed	=> '$Date: 2005-12-11 19:45:54 -0300 (dom, 11 dic 2005) $',
    changes	=> "see ChangeLog"
);


use FileHandle;
use Irssi::TextUI;

sub trimString {
    my ($line) = @_;
    chomp($_ = $line);
    s/[\t\s\n\r\f]*$//gim;
    s/^[\t\s\n\r\f]*//gim;
    return $_;
}

sub cleanWhiteSpace {
    my ($string) = @_;
    $_ = $string;
    s/\s(?=\s)//gim; s/\t(?=\t)//gim;
    return $_;
}

sub isEmpty {
    my ($obj) = @_;
    if ((!defined($obj)) || (trimStr($obj) ne "") || ($obj != 0)) {
        return 0;
    } else {
        return 1;
    }
}

sub xmmsInfo2Hash {
	my ($xi, $buffer, $line, @lines, %ihash);
	my ($c, $wnd);
	$xi = new FileHandle("/tmp/xmms-info");
	if (defined $xi) {
		@lines = $xi->getlines;
		$xi->close;
	} else {
		return 0;
	}
	$c = 0;
	foreach $line (@lines) {
		$lines[$c] = trimString($line);
		$c++;
	}
	foreach $line (@lines) {
		if ($line =~ /^Status:/) {
			$line = trimString($line);
			$line =~ s/^Status:[\t\s]+//gim;
			$buffer = $line;
			$ihash{'status'} = $buffer;
		}
		if ($line =~ /^Tunes in playlist:/) {
			$line = trimString($line);
			$line =~ s/^Tunes in playlist:[\t\s]+//gim;
			$buffer = $line;
			$ihash{'songs'} = $buffer;
		}
		if ($line =~ /^Currently playing:/) {
			$line = trimString($line);
			$line =~ s/^Currently playing:[\t\s]+//gim;
			$buffer = $line;
			$ihash{'current'} = $buffer;
		}
		if ($line =~ /^Position:/) {
			$line = trimString($line);
			$line =~ s/^Position:[\t\s]+//gim;
			$buffer = $line;
			$ihash{'position'} = $buffer;
		}
		if ($line =~ /^Time:/) {
			$line = trimString($line);
			$line =~ s/^Time:[\t\s]+//gim;
			$buffer = $line;
			$ihash{'time'} = $buffer;
		}
		if ($line =~ /^Title:/) {
			$line = trimString($line);
			$line =~ s/^Title:[\t\s]+//gim;
			$buffer = xmmsProcessTitle($line);
			$ihash{'title'} = $buffer;
		}
		if ($line =~ /^Current bitrate:/) {
			$line = trimString($line);
			$line =~ s/^Current bitrate:[\t\s]+//gim;
			$buffer = trimString($line);
			$ihash{'bitrate'} = (int($buffer)/1000);
		}
		if ($line =~ /^Samping Frequency:/) {
			$line = trimString($line);
			$line =~ s/^Samping Frequency:[\t\s]+//gim;
			$buffer = trimString($line);
			$ihash{'freq'} = sprintf("%.2f", ($buffer/1000));
		}
		if ($line =~ /^uSecPosition:/) {
			$line = trimString($line);
			$line =~ s/^uSecPosition:[\t\s]+//gim;
			$buffer = trimString($line);
			$ihash{'pos'} = $buffer;
		}
		if ($line =~ /^uSecTime:/) {
			$line = trimString($line);
			$line =~ s/^uSecTime:[\t\s]+//gim;
			$buffer = trimString($line);
			$ihash{'total'} = $buffer;
		}
	}
	return %ihash;
}

sub xmmsProcessTitle {
	my ($title) = @_;
	my ($album, $artist, $song);
	my ($ptitle);
	if ($title =~ /.+\|.+\|.+/) {
		($artist, $album, $song) = split(/\|/, $title);
		$album = cleanWhiteSpace(trimString($album));
		$artist = cleanWhiteSpace(trimString($artist));
		$song = cleanWhiteSpace(trimString($song));
		$ptitle = "\002$artist\002: \"$song\" [Album: \002$album\002]";
		return $ptitle;
	} elsif ($title =~ /.+\|[\s\t]*\|.+/) {
		($artist, $album, $song) = split(/\|/, $title);
		$album = cleanWhiteSpace(trimString($album));
		$artist = cleanWhiteSpace(trimString($artist));
		$song = cleanWhiteSpace(trimString($song));
		$ptitle = "\002$artist\002: \"$song\"";
		return $ptitle;
	} elsif ($title =~ /.+-.+/) {
		($artist, $song) = split(/\-/, $title);
		$album = cleanWhiteSpace(trimString($album));
		$artist = cleanWhiteSpace(trimString($artist));
		$ptitle = "\002$artist\002: \"$song\"";
		return $ptitle;
	} else {
		$ptitle = trimString($title);
		$ptitle = "\"$ptitle\"";
		return $ptitle;
	}
}

sub cmdMp3Info {
	my (%ihash) = xmmsInfo2Hash();
	my ($wnd) = Irssi::active_win();
	my ($song);
	my ($percent);
	$song  = "me with XMMS " . $ihash{'status'};
	$song .= " " . $ihash{'title'};
	if ($ihash{'pos'} > 0 && $ihash{'total'} > 0) {
		$percent = $ihash{'pos'} * 100 / $ihash{'total'};
		$song .= " [" . $ihash{'position'} . "/" . $ihash{'time'} . ":";
		$song .= sprintf("%3.0d%%", $percent) . "]";
	} else {
		$song .= " [" . $ihash{'time'} . "/" . $ihash{'position'} . "]";
	}
	$song .= " [" . $ihash{'bitrate'} . "K/" . $ihash{'freq'} . "kHz]";
	$song .= " [Track:" . $ihash{'current'} . "/" . $ihash{'songs'} . "]";
	$song .= " [\002Irssi\@FreeBSD\002]";
	$wnd->command($song);
}

Irssi::command_bind('mp3info', 'cmdMp3Info');

# vim: set ft=perl ff=unix ts=4 sw=4 enc=latin1 expandtab:
